/*********************************************************************
 * Libreria para la gestion de motores paso a paso "Unipolares" con 
 * Arduino, con solo indicar los pines donde se conecta, nos abstrae
 * del resto de funciones pudiendo realizar las siguientes operaciones:
 *      avanzar tanto en paso como por angulo.
 *      retroceder de la misma forma que avance.
 *      controlar la velocidad de avance.
 *      
 * Autor:   Fco. Javier Rodriguez Navarro   www.pinguytaz.net
 * Version: 1.0 Oct. 2016
 *    
 * DATOS ALMACENAMIENTO:      
 * Clase 918 bytes  V. Globales 167 bytes
 *********************************************************************/


#include <Arduino.h> //Librerias de Arduino.
#include "MPaP.h"

 
/*********************************************************************
 * Constructores de la clase
 * 
 * MPaP(Pin_BobinaA,Pin_BobinaB,Pin_BobinaC,Pin_BobinaD)
 * Parametros: 
 *  Los pines a las bobinas del motor, por defecto las variables pps, 
 *  rpm, angulo, pasos, pvuelta son las definidas en MPaP.h.
 *  
 * MPaP::MPaP(int a, int b, int c, int d, int pas)
 * Parametros: 
 *  Igual que el anterior pero en este caso definimos los pasos 
 *  necesarios para dar una vuelta "pvuelta"
 *  
 * Los parametros de las salidas digitales de las bobinas
 *   A
 *  D  B
 *   C
 ************************************************************/
MPaP::MPaP(int a, int b, int c, int d)
{ 
   asignapines(a,b,c,d);   
}

MPaP::MPaP(int a, int b, int c, int d, int pas)
{
   pvuelta = pas;
   angulo = 360/pvuelta;

   asignapines(a,b,c,d);
}

MPaP::~MPaP(){}


// Privada: que asigna los pines definidos como salida

void MPaP::asignapines(int a, int b, int c, int d)
{

   paso = -1;
  
   BOB_A = a;
   BOB_B = b;
   BOB_C = c;
   BOB_D = d;
   pinMode( BOB_A , OUTPUT); 
   pinMode( BOB_B , OUTPUT); 
   pinMode( BOB_C , OUTPUT); 
   pinMode( BOB_D , OUTPUT); 
}

/***************   Metodos para variables *******************
 * secuencia: NORMAL, WAVE, MEDIO Como se mueve cada paso.
 * pasos: Los pasos que se dan en una funcion de avance o retroceso.
 * rpm: Revoluciones por minuto a la que da los pasos, es mayor al maximo da el maximo
 * pps: Maximos pasos por segundo que admite el motor.
 * angulo: Los angulos que se dan en cada paso
 * pvuelta: Los pasos de una vuelta.
 * 
 ************************************************************/
void MPaP::put_secuencia(int a) { secuencia = a; }
int MPaP::get_secuencia(void) { return secuencia; }
void MPaP::put_pasos(int a) { pasos = a; }
int MPaP::get_pasos(void) { return pasos; }
float  MPaP::put_rpm(float r) 
{
     if (r > max_rpm) rpm = max_rpm; 
     else rpm = r;
     
     return rpm;
 }
float  MPaP::put_rpm() { rpm = max_rpm; return rpm;}
float MPaP::get_rpm(void) { return rpm; }
void MPaP::put_pps(int p)
{ 
  pps = p; 
  max_rpm = pps*60/pvuelta;  
  if (rpm > max_rpm) rpm = max_rpm;
}
int MPaP::get_pps(void) { return pps; }
void MPaP::put_angulo(float a) { angulo = a; pvuelta = 360/angulo; }
float MPaP::get_angulo(void) { return angulo; }
void MPaP::put_pvuelta(int pv) { pvuelta = pv; angulo = 360/pvuelta; }
int MPaP::get_pvuelta(void) { return pvuelta; }


/***************** METODOS publicos ****************************/
void MPaP::avanza(void) // Avanza el numero de pasos indicado en "pasos"
{ 
   avanza(pasos);
}

void MPaP::avanza(int p) // Avanza el numero de pasos indicado en el parametro
{
   for (int x = 0; x < p; x++) avanza_uno();
}

void MPaP::retrocede(void) // Retrocede el numero de pasos indicado en "pasos"
{ 
   retrocede(pasos);
}

void MPaP::retrocede(int p) // Retrocede el numero de pasos indicado en el parametro
{
   for (int x = 0; x < p; x++) retrocede_uno();
}

void MPaP::angulo_a(float ang) // Avanza los grados indicados en el parametro.
{ 
  avanza(cpasos(ang)); 
}
void MPaP::angulo_r(float ang) // Retrocede los grados indicados en el parametro.
{ 
  retrocede(cpasos(ang)); 
}
 


/***************** METODOS privados ****************************/
int MPaP::cpasos(float a)  // Calcula los pasos a realizar segun el angulo pasado, la secuencia programada y el angulo por paso
{
  if (secuencia == MEDIO) return (a/(angulo/2));
  else return (a/angulo);
}
void MPaP::avanza_uno(void) // Realiza un paso y actualiza el numero de secuencia.
{ 
   paso++;
 
   if(secuencia != MEDIO && paso > 3) paso = 0;
   else if (paso > 7) paso = 0;
   activa();
}
void MPaP::retrocede_uno(void) // Retrocede un paso y actualiza el numero de secuencia.
{ 
    paso--;
    if(paso < 0)
    {
      if (secuencia == MEDIO) paso = 7;
      else paso = 3;
    }
    else if (paso > 7) paso = 0;

     activa();
 }

 void MPaP::activa(void)  // Acciona los pines segun la secuencia y realiza un retrardo segun rpm.
 {
   int a,b,c,d;

  
     switch(secuencia)
     {
      case NORMAL:
         a=sec_NORMAL[paso][0];
         b=sec_NORMAL[paso][1];
         c=sec_NORMAL[paso][2];
         d=sec_NORMAL[paso][3];
         break;
      case WAVE:
         a=sec_WAVE[paso][0];
         b=sec_WAVE[paso][1];
         c=sec_WAVE[paso][2];
         d=sec_WAVE[paso][3];
         break;
      case MEDIO:
         a=sec_MEDIO[paso][0];
         b=sec_MEDIO[paso][1];
         c=sec_MEDIO[paso][2];
         d=sec_MEDIO[paso][3];
         break;
     }

     digitalWrite( BOB_A, a );
     digitalWrite( BOB_B, b );
     digitalWrite( BOB_C, c );
     digitalWrite( BOB_D, d );
     
     if(secuencia == MEDIO) delay(60/(rpm*pvuelta*2)*1000);  // Retardo para dar la velocidad RPM en opcion medio
     else delay(60/(rpm*pvuelta)*1000);  // Retardo para dar la velocidad RPM
  }


